@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
+ */
+
 /*
  * squid_ldap_auth: authentication via ldap for squid proxy server
  *
@@ -89,12 +97,10 @@
 #include "rfc1738.h"
 #include "util.h"
 
-#include <stdio.h>
-#include <string.h>
-#include <ctype.h>
-
-#if _SQUID_MSWIN_		/* Native Windows port and MinGW */
+#include <cctype>
+#include <cstring>
 
+#if _SQUID_WINDOWS_ && !_SQUID_CYGWIN_
 #define snprintf _snprintf
 #include <windows.h>
 #include <winldap.h>
@@ -267,7 +273,7 @@ open_ldap_connection(const char *ldapServer, int port)
                         sslpath);
                 exit(1);
             } else {
-                sslinit++;
+                ++sslinit;
             }
             if ((ld = ldapssl_init(ldapServer, port, 1)) == NULL) {
                 fprintf(stderr, "\nUnable to connect to SSL LDAP server: %s port:%d\n",
@@ -332,7 +338,7 @@ validUsername(const char *user)
             if (p[0] != ' ')
                 return 0;
         }
-        p++;
+        ++p;
     }
     /* Trailing whitespace? */
     if (xisspace(p[0]))
@@ -368,14 +374,14 @@ main(int argc, char **argv)
                 value = argv[1] + 2;
             } else if (argc > 2) {
                 value = argv[2];
-                argv++;
-                argc--;
+                ++argv;
+                --argc;
             } else
                 value = "";
             break;
         }
-        argv++;
-        argc--;
+        ++argv;
+        --argc;
         switch (option) {
         case 'H':
 #if !HAS_URI_SUPPORT
@@ -513,8 +519,8 @@ main(int argc, char **argv)
         } else {
             ldapServer = xstrdup(value);
         }
-        argc--;
-        argv++;
+        --argc;
+        ++argv;
     }
     if (!ldapServer)
         ldapServer = xstrdup("localhost");
@@ -554,7 +560,7 @@ main(int argc, char **argv)
     /* On Windows ldap_start_tls_s is available starting from Windows XP,
      * so we need to bind at run-time with the function entry point
      */
-#if _SQUID_MSWIN_
+#if _SQUID_WINDOWS_
     if (use_tls) {
 
         HMODULE WLDAP32Handle;
