@@ -1,8 +1,13 @@
 /*
- * DEBUG: section 16    Cache Manager API
+ * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
  *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
  */
 
+/* DEBUG: section 16    Cache Manager API */
+
 #include "squid.h"
 #include "base/TextException.h"
 #include "ipc/Messages.h"
@@ -13,7 +18,7 @@
 
 Mgr::StoreIoActionData::StoreIoActionData()
 {
-    xmemset(this, 0, sizeof(*this));
+    memset(this, 0, sizeof(*this));
 }
 
 Mgr::StoreIoActionData&
@@ -33,8 +38,8 @@ Mgr::StoreIoAction::Create(const CommandPointer &cmd)
     return new StoreIoAction(cmd);
 }
 
-Mgr::StoreIoAction::StoreIoAction(const CommandPointer &cmd):
-        Action(cmd), data()
+Mgr::StoreIoAction::StoreIoAction(const CommandPointer &aCmd):
+        Action(aCmd), data()
 {
     debugs(16, 5, HERE);
 }
